Method: Test::Unit::TestCase#data_label
- Defined in:
- lib/test/unit/testcase.rb
#data_label ⇒ Object
Returns a label of test data for the test. If the test isn’t associated with any test data, it returns nil.
490 491 492 |
# File 'lib/test/unit/testcase.rb', line 490 def data_label @internal_data.test_data_label end |