Class: Test::Unit::AutoRunner

Inherits:
Object
  • Object
show all
Defined in:
lib/test/unit/autorunner.rb

Constant Summary collapse

RUNNERS =
{}
COLLECTORS =
{}
ADDITIONAL_OPTIONS =
[]
PREPARE_HOOKS =
[]
@@default_runner =
nil
@@need_auto_run =
true

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(standalone) {|_self| ... } ⇒ AutoRunner

Returns a new instance of AutoRunner.

Yields:

  • (_self)

Yield Parameters:



143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
# File 'lib/test/unit/autorunner.rb', line 143

def initialize(standalone)
  @standalone = standalone
  @runner = default_runner
  @collector = default_collector
  @filters = []
  @to_run = []
  @default_test_paths = []
  @color_scheme = ColorScheme.default
  @runner_options = {}
  @default_arguments = []
  @workdir = nil
  @listeners = []
  config_file = "test-unit.yml"
  if File.exist?(config_file)
    load_config(config_file)
  else
    load_global_config
  end
  yield(self) if block_given?
end

Instance Attribute Details

#baseObject

Returns the value of attribute base.



139
140
141
# File 'lib/test/unit/autorunner.rb', line 139

def base
  @base
end

#collector=(value) ⇒ Object (writeonly)

Sets the attribute collector

Parameters:

  • value

    the value to set the attribute collector to.



141
142
143
# File 'lib/test/unit/autorunner.rb', line 141

def collector=(value)
  @collector = value
end

#color_schemeObject

Returns the value of attribute color_scheme.



140
141
142
# File 'lib/test/unit/autorunner.rb', line 140

def color_scheme
  @color_scheme
end

#default_test_pathsObject

Returns the value of attribute default_test_paths.



138
139
140
# File 'lib/test/unit/autorunner.rb', line 138

def default_test_paths
  @default_test_paths
end

#excludeObject

Returns the value of attribute exclude.



139
140
141
# File 'lib/test/unit/autorunner.rb', line 139

def exclude
  @exclude
end

#filtersObject

Returns the value of attribute filters.



137
138
139
# File 'lib/test/unit/autorunner.rb', line 137

def filters
  @filters
end

#listenersObject

Returns the value of attribute listeners.



140
141
142
# File 'lib/test/unit/autorunner.rb', line 140

def listeners
  @listeners
end

#patternObject

Returns the value of attribute pattern.



139
140
141
# File 'lib/test/unit/autorunner.rb', line 139

def pattern
  @pattern
end

#runner=(value) ⇒ Object (writeonly)

Sets the attribute runner

Parameters:

  • value

    the value to set the attribute runner to.



141
142
143
# File 'lib/test/unit/autorunner.rb', line 141

def runner=(value)
  @runner = value
end

#runner_optionsObject (readonly)

Returns the value of attribute runner_options.



136
137
138
# File 'lib/test/unit/autorunner.rb', line 136

def runner_options
  @runner_options
end

#suiteObject (readonly)

Returns the value of attribute suite.



136
137
138
# File 'lib/test/unit/autorunner.rb', line 136

def suite
  @suite
end

#to_runObject

Returns the value of attribute to_run.



137
138
139
# File 'lib/test/unit/autorunner.rb', line 137

def to_run
  @to_run
end

#workdirObject

Returns the value of attribute workdir.



139
140
141
# File 'lib/test/unit/autorunner.rb', line 139

def workdir
  @workdir
end

Class Method Details

.collector(id) ⇒ Object



40
41
42
# File 'lib/test/unit/autorunner.rb', line 40

def collector(id)
  COLLECTORS[id.to_s]
end

.default_runnerObject



27
28
29
# File 'lib/test/unit/autorunner.rb', line 27

def default_runner
  runner(@@default_runner)
end

.default_runner=(id) ⇒ Object



31
32
33
# File 'lib/test/unit/autorunner.rb', line 31

def default_runner=(id)
  @@default_runner = id
end

.need_auto_run=(need) ⇒ Object



77
78
79
# File 'lib/test/unit/autorunner.rb', line 77

def need_auto_run=(need)
  @@need_auto_run = need
end

.need_auto_run?Boolean

Returns:

  • (Boolean)


73
74
75
# File 'lib/test/unit/autorunner.rb', line 73

def need_auto_run?
  @@need_auto_run
end

.prepare(hook = Proc.new) ⇒ Object



52
53
54
# File 'lib/test/unit/autorunner.rb', line 52

def prepare(hook=Proc.new)
  PREPARE_HOOKS << hook
end

.register_collector(id, collector_builder = Proc.new) ⇒ Object



35
36
37
38
# File 'lib/test/unit/autorunner.rb', line 35

def register_collector(id, collector_builder=Proc.new)
  COLLECTORS[id] = collector_builder
  COLLECTORS[id.to_s] = collector_builder
end

.register_color_scheme(id, scheme) ⇒ Object



44
45
46
# File 'lib/test/unit/autorunner.rb', line 44

def register_color_scheme(id, scheme)
  ColorScheme[id] = scheme
end

.register_runner(id, runner_builder = Proc.new) ⇒ Object



17
18
19
20
# File 'lib/test/unit/autorunner.rb', line 17

def register_runner(id, runner_builder=Proc.new)
  RUNNERS[id] = runner_builder
  RUNNERS[id.to_s] = runner_builder
end

.run(force_standalone = false, default_dir = nil, argv = ARGV, &block) ⇒ Object



56
57
58
59
60
61
62
# File 'lib/test/unit/autorunner.rb', line 56

def run(force_standalone=false, default_dir=nil, argv=ARGV, &block)
  r = new(force_standalone || standalone?, &block)
  r.base = default_dir
  r.prepare
  r.process_args(argv)
  r.run
end

.runner(id) ⇒ Object



22
23
24
# File 'lib/test/unit/autorunner.rb', line 22

def runner(id)
  RUNNERS[id.to_s]
end

.setup_option(option_builder = Proc.new) ⇒ Object



48
49
50
# File 'lib/test/unit/autorunner.rb', line 48

def setup_option(option_builder=Proc.new)
  ADDITIONAL_OPTIONS << option_builder
end

.standalone?Boolean

Returns:

  • (Boolean)


64
65
66
67
68
69
70
# File 'lib/test/unit/autorunner.rb', line 64

def standalone?
  return false unless("-e" == $0)
  ObjectSpace.each_object(Class) do |klass|
    return false if(klass < TestCase)
  end
  true
end

Instance Method Details

#keyword_display(keywords) ⇒ Object



393
394
395
396
397
398
399
400
401
402
403
404
405
406
407
408
409
# File 'lib/test/unit/autorunner.rb', line 393

def keyword_display(keywords)
  keywords = keywords.collect do |keyword, _|
    keyword.to_s
  end.uniq.sort

  i = 0
  keywords.collect do |keyword|
    if (i > 0 and keyword[0] == keywords[i - 1][0]) or
        ((i < keywords.size - 1) and (keyword[0] == keywords[i + 1][0]))
      n = 2
    else
      n = 1
    end
    i += 1
    keyword.sub(/^(.{#{n}})([A-Za-z]+)(?=\w*$)/, '\\1[\\2]')
  end.join(", ")
end

#load_config(file) ⇒ Object



426
427
428
429
430
431
432
433
434
435
436
437
438
439
440
441
442
443
444
445
446
# File 'lib/test/unit/autorunner.rb', line 426

def load_config(file)
  require 'yaml'
  config = YAML.load(File.read(file))
  runner_name = config["runner"]
  @runner = self.class.runner(runner_name) || @runner
  @collector = self.class.collector(config["collector"]) || @collector
  (config["color_schemes"] || {}).each do |name, options|
    ColorScheme[name] = options
  end
  runner_options = {}
  (config["#{runner_name}_options"] || {}).each do |key, value|
    key = key.to_sym
    value = ColorScheme[value] if key == :color_scheme
    if key == :arguments
      @default_arguments.concat(value.split)
    else
      runner_options[key.to_sym] = value
    end
  end
  @runner_options = @runner_options.merge(runner_options)
end

#optionsObject



182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
316
317
318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
391
# File 'lib/test/unit/autorunner.rb', line 182

def options
  @options ||= OptionParser.new do |o|
    o.banner = "Test::Unit automatic runner."
    o.banner << "\nUsage: #{$0} [options] [-- untouched arguments]"

    o.on('-r', '--runner=RUNNER', RUNNERS,
         "Use the given RUNNER.",
         "(" + keyword_display(RUNNERS) + ")") do |r|
      @runner = r
    end

    o.on('--collector=COLLECTOR', COLLECTORS,
         "Use the given COLLECTOR.",
         "(" + keyword_display(COLLECTORS) + ")") do |collector|
      @collector = collector
    end

    if (@standalone)
      o.on('-b', '--basedir=DIR', "Base directory of test suites.") do |b|
        @base = b
      end

      o.on('-w', '--workdir=DIR', "Working directory to run tests.") do |w|
        @workdir = w
      end

      o.on('--default-test-path=PATH',
           "Add PATH to the default test paths.",
           "The PATH is used when user doesn't specify any test path.",
           "You can specify this option multiple times.") do |path|
        @default_test_paths << path
      end

      o.on('-a', '--add=TORUN', Array,
           "Add TORUN to the list of things to run;",
           "can be a file or a directory.") do |paths|
        paths.each do |path|
          add_test_path(path)
        end
      end

      @pattern = []
      o.on('-p', '--pattern=PATTERN', Regexp,
           "Match files to collect against PATTERN.") do |e|
        @pattern << e
      end

      @exclude = []
      o.on('-x', '--exclude=PATTERN', Regexp,
           "Ignore files to collect against PATTERN.") do |e|
        @exclude << e
      end
    end

    o.on('-n', '--name=NAME', String,
         "Runs tests matching NAME.",
         "Use '/PATTERN/' for NAME to use regular expression.") do |name|
      name = (%r{\A/(.*)/\Z} =~ name ? Regexp.new($1) : name)
      @filters << lambda do |test|
        return true if name === test.method_name
        test_name_without_class_name = test.name.gsub(/\(.+?\)\z/, "")
        if test_name_without_class_name != test.method_name
          return true if name === test_name_without_class_name
        end
        false
      end
    end

    o.on('--ignore-name=NAME', String,
         "Ignores tests matching NAME.",
         "Use '/PATTERN/' for NAME to use regular expression.") do |n|
      n = (%r{\A/(.*)/\Z} =~ n ? Regexp.new($1) : n)
      case n
      when Regexp
        @filters << proc {|t| n =~ t.method_name ? false : true}
      else
        @filters << proc {|t| n != t.method_name}
      end
    end

    o.on('-t', '--testcase=TESTCASE', String,
         "Runs tests in TestCases matching TESTCASE.",
         "Use '/PATTERN/' for TESTCASE to use regular expression.") do |n|
      n = (%r{\A/(.*)/\Z} =~ n ? Regexp.new($1) : n)
      @filters << lambda do |test|
        match_test_case_name(test, n)
      end
    end

    o.on('--ignore-testcase=TESTCASE', String,
         "Ignores tests in TestCases matching TESTCASE.",
         "Use '/PATTERN/' for TESTCASE to use regular expression.") do |n|
      n = (%r{\A/(.*)/\Z} =~ n ? Regexp.new($1) : n)
      @filters << lambda do |test|
        not match_test_case_name(test, n)
      end
    end

    o.on('--location=LOCATION', String,
         "Runs tests that defined in LOCATION.",
         "LOCATION is one of PATH:LINE, PATH or LINE") do |location|
      if /\A\d+\z/ =~ location
        path = nil
        line = location.to_i
      else
        path, line, = location.split(/:(\d+)/, 2)
        line = line.to_i unless line.nil?
      end
      add_location_filter(path, line)
    end

    o.on('--attribute=EXPRESSION', String,
         "Runs tests that matches EXPRESSION.",
         "EXPRESSION is evaluated as Ruby's expression.",
         "Test attribute name can be used with no receiver in EXPRESSION.",
         "EXPRESSION examples:",
         "  !slow",
         "  tag == 'important' and !slow") do |expression|
      @filters << lambda do |test|
        matcher = AttributeMatcher.new(test)
        matcher.match?(expression)
      end
    end

    priority_filter = Proc.new do |test|
      if @filters == [priority_filter]
        Priority::Checker.new(test).need_to_run?
      else
        nil
      end
    end
    o.on("--[no-]priority-mode",
         "Runs some tests based on their priority.") do |priority_mode|
      if priority_mode
        Priority.enable
        @filters |= [priority_filter]
      else
        Priority.disable
        @filters -= [priority_filter]
      end
    end

    o.on("--default-priority=PRIORITY",
         Priority.available_values,
         "Uses PRIORITY as default priority",
         "(#{keyword_display(Priority.available_values)})") do |priority|
      Priority.default = priority
    end

    o.on('-I', "--load-path=DIR[#{File::PATH_SEPARATOR}DIR...]",
         "Appends directory list to $LOAD_PATH.") do |dirs|
      $LOAD_PATH.concat(dirs.split(File::PATH_SEPARATOR))
    end

    color_schemes = ColorScheme.all
    o.on("--color-scheme=SCHEME", color_schemes,
         "Use SCHEME as color scheme.",
         "(#{keyword_display(color_schemes)})") do |scheme|
      @color_scheme = scheme
    end

    o.on("--config=FILE",
         "Use YAML fomat FILE content as configuration file.") do |file|
      load_config(file)
    end

    o.on("--order=ORDER", TestCase::AVAILABLE_ORDERS,
         "Run tests in a test case in ORDER order.",
         "(#{keyword_display(TestCase::AVAILABLE_ORDERS)})") do |order|
      TestCase.test_order = order
    end

    assertion_message_class = Test::Unit::Assertions::AssertionMessage
    o.on("--max-diff-target-string-size=SIZE", Integer,
         "Shows diff if both expected result string size and " +
         "actual result string size are " +
         "less than or equal SIZE in bytes.",
         "(#{assertion_message_class.max_diff_target_string_size})") do |size|
      assertion_message_class.max_diff_target_string_size = size
    end

    ADDITIONAL_OPTIONS.each do |option_builder|
      option_builder.call(self, o)
    end

    o.on('--',
         "Stop processing options so that the",
         "remaining options will be passed to the",
         "test."){o.terminate}

    o.on('-h', '--help', 'Display this help.'){puts o; exit}

    o.on_tail
    o.on_tail('Deprecated options:')

    o.on_tail('--console', 'Console runner (use --runner).') do
      warn("Deprecated option (--console).")
      @runner = self.class.runner(:console)
    end

    if RUNNERS[:fox]
      o.on_tail('--fox', 'Fox runner (use --runner).') do
        warn("Deprecated option (--fox).")
        @runner = self.class.runner(:fox)
      end
    end

    o.on_tail
  end
end

#prepareObject



164
165
166
167
168
# File 'lib/test/unit/autorunner.rb', line 164

def prepare
  PREPARE_HOOKS.each do |handler|
    handler.call(self)
  end
end

#process_args(args = ARGV) ⇒ Object



170
171
172
173
174
175
176
177
178
179
180
# File 'lib/test/unit/autorunner.rb', line 170

def process_args(args=ARGV)
  begin
    args.unshift(*@default_arguments)
    options.order!(args) {|arg| add_test_path(arg)}
  rescue OptionParser::ParseError => e
    puts e
    puts options
    exit(false)
  end
  not @to_run.empty?
end

#runObject



411
412
413
414
415
416
417
418
419
420
421
422
423
424
# File 'lib/test/unit/autorunner.rb', line 411

def run
  self.class.need_auto_run = false
  suite = @collector[self]
  return false if suite.nil?
  return true if suite.empty?
  runner = @runner[self]
  return false if runner.nil?
  @runner_options[:color_scheme] ||= @color_scheme
  @runner_options[:listeners] ||= []
  @runner_options[:listeners].concat(@listeners)
  change_work_directory do
    runner.run(suite, @runner_options).passed?
  end
end