Method: Test::Unit::TestCase.suite
- Defined in:
- lib/test/unit/testcase.rb
.suite ⇒ Object
Rolls up all of the test* methods in the fixture into one suite, creating a new instance of the fixture for each method.
128 129 130 131 |
# File 'lib/test/unit/testcase.rb', line 128 def suite suite_creator = TestSuiteCreator.new(self) suite_creator.create end |