Class: Test::Unit::TestCase
- Inherits:
-
Object
- Object
- Test::Unit::TestCase
- Includes:
- Assertions, Attribute, ErrorHandler, ExceptionHandler, FailureHandler, Fixture, Priority, TestCaseNotificationSupport, TestCaseOmissionSupport, TestCasePendingSupport, Util::BacktraceFilter
- Defined in:
- lib/test/unit/testcase.rb
Overview
Ties everything together. If you subclass and add your own test methods, it takes care of making them into tests and wrapping those tests into a suite. It also does the nitty-gritty of actually running an individual test and collecting its results into a Test::Unit::TestResult object.
You can run two hooks before/after a TestCase run.
Example:
class TestMyClass < Test::Unit::TestCase
class << self
def startup
...
end
def shutdown
...
end
end
def setup
...
end
def teardown
...
end
def test_my_method1
...
end
def test_my_method2
...
end
end
Here is a call order:
-
startup
-
setup
-
test_my_method1
-
teardown
-
setup
-
test_my_method2
-
teardown
-
shutdown
Constant Summary collapse
- STARTED =
:nodoc:
name + "::STARTED"
- FINISHED =
:nodoc:
name + "::FINISHED"
- DESCENDANTS =
:nodoc:
[]
- AVAILABLE_ORDERS =
:nodoc:
[:alphabetic, :random, :defined]
- @@added_methods =
[]
- @@test_order =
AVAILABLE_ORDERS.first
Constants included from Util::BacktraceFilter
Util::BacktraceFilter::TESTUNIT_FILE_SEPARATORS, Util::BacktraceFilter::TESTUNIT_PREFIX, Util::BacktraceFilter::TESTUNIT_RB_FILE
Constants included from Assertions
Constants included from ErrorHandler
ErrorHandler::PASS_THROUGH_EXCEPTIONS
Instance Attribute Summary collapse
-
#method_name ⇒ Object
readonly
Returns the value of attribute method_name.
Class Method Summary collapse
-
.inherited(sub_class) ⇒ Object
:nodoc:.
-
.method_added(name) ⇒ Object
:nodoc:.
-
.shutdown ⇒ Object
Called after every test case runs.
-
.startup ⇒ Object
Called before every test case runs.
-
.suite ⇒ Object
Rolls up all of the test* methods in the fixture into one suite, creating a new instance of the fixture for each method.
-
.test_order ⇒ Object
Returns the current test order.
-
.test_order=(order) ⇒ Object
Sets the current test order.
Instance Method Summary collapse
-
#==(other) ⇒ Object
It’s handy to be able to compare TestCase instances.
- #default_test ⇒ Object
-
#initialize(test_method_name) ⇒ TestCase
constructor
Creates a new instance of the fixture for running the test represented by test_method_name.
- #interrupted? ⇒ Boolean
-
#name ⇒ Object
Returns a human-readable name for the specific test that this instance of TestCase represents.
-
#run(result) ⇒ Object
Runs the individual test method represented by this instance of the fixture, collecting statistics, failures and errors in result.
-
#setup ⇒ Object
Called before every test method runs.
- #size ⇒ Object
-
#teardown ⇒ Object
Called after every test method runs.
-
#to_s ⇒ Object
Overridden to return #name.
Methods included from Util::BacktraceFilter
Methods included from Assertions
#assert, #assert_block, #assert_boolean, #assert_compare, #assert_const_defined, #assert_equal, #assert_fail_assertion, #assert_false, #assert_in_delta, #assert_instance_of, #assert_kind_of, #assert_match, #assert_nil, #assert_no_match, #assert_not_const_defined, #assert_not_equal, #assert_not_nil, #assert_not_predicate, #assert_not_same, #assert_nothing_raised, #assert_nothing_thrown, #assert_operator, #assert_predicate, #assert_raise, #assert_raise_kind_of, #assert_raise_message, #assert_raises, #assert_respond_to, #assert_same, #assert_send, #assert_throw, #assert_throws, #assert_true, #build_message, #flunk, use_pp=
Methods included from Priority
available_values, default, default=, disable, enable, enabled?, included, #priority_setup, #priority_teardown
Methods included from TestCaseNotificationSupport
Methods included from TestCaseOmissionSupport
included, #omit, #omit_if, #omit_unless
Methods included from TestCasePendingSupport
Methods included from FailureHandler
Methods included from ErrorHandler
Methods included from ExceptionHandler
Methods included from Fixture
Methods included from Attribute
Constructor Details
#initialize(test_method_name) ⇒ TestCase
Creates a new instance of the fixture for running the test represented by test_method_name.
257 258 259 260 261 262 263 264 265 266 267 268 |
# File 'lib/test/unit/testcase.rb', line 257 def initialize(test_method_name) throw :invalid_test unless respond_to?(test_method_name) test_method = method(test_method_name) throw :invalid_test if test_method.arity > 0 owner = Util::MethodOwnerFinder.find(self, test_method_name) if owner.class != Module and self.class != owner throw :invalid_test end @method_name = test_method_name @test_passed = true @interrupted = false end |
Instance Attribute Details
#method_name ⇒ Object (readonly)
Returns the value of attribute method_name.
253 254 255 |
# File 'lib/test/unit/testcase.rb', line 253 def method_name @method_name end |
Class Method Details
.inherited(sub_class) ⇒ Object
:nodoc:
93 94 95 |
# File 'lib/test/unit/testcase.rb', line 93 def inherited(sub_class) # :nodoc: DESCENDANTS << sub_class end |
.method_added(name) ⇒ Object
:nodoc:
98 99 100 101 |
# File 'lib/test/unit/testcase.rb', line 98 def method_added(name) # :nodoc: super @@added_methods << name.to_s end |
.shutdown ⇒ Object
Called after every test case runs. Can be used to tear down fixture information used in test case scope.
Here is an example test case:
class TestMyClass < Test::Unit::TestCase
class << self
def shutdown
...
end
end
def teardown
...
end
def test_my_class1
...
end
def test_my_class2
...
end
end
Here is a call order:
-
test_my_class1 (or test_my_class2)
-
teardown
-
test_my_class2 (or test_my_class1)
-
teardown
-
shutdown
Note that you should not assume test order. Tests should be worked in any order.
191 192 |
# File 'lib/test/unit/testcase.rb', line 191 def shutdown end |
.startup ⇒ Object
Called before every test case runs. Can be used to set up fixture information used in test case scope.
Here is an example test case:
class TestMyClass < Test::Unit::TestCase
class << self
def startup
...
end
end
def setup
...
end
def test_my_class1
...
end
def test_my_class2
...
end
end
Here is a call order:
-
startup
-
setup
-
test_my_class1 (or test_my_class2)
-
setup
-
test_my_class2 (or test_my_class1)
Note that you should not assume test order. Tests should be worked in any order.
155 156 |
# File 'lib/test/unit/testcase.rb', line 155 def startup end |
.suite ⇒ Object
Rolls up all of the test* methods in the fixture into one suite, creating a new instance of the fixture for each method.
106 107 108 109 110 111 112 113 114 115 116 117 118 119 |
# File 'lib/test/unit/testcase.rb', line 106 def suite suite = TestSuite.new(name, self) collect_test_names.each do |test| catch(:invalid_test) do suite << new(test) end end if suite.empty? catch(:invalid_test) do suite << new("default_test") end end suite end |
.test_order ⇒ Object
Returns the current test order. This returns :alphabetic by default.
198 199 200 |
# File 'lib/test/unit/testcase.rb', line 198 def test_order @@test_order end |
.test_order=(order) ⇒ Object
Sets the current test order.
Here are the available order:
- :alphabetic
-
Default. Tests are sorted in alphabetic order.
- :random
-
Tests are sorted in random order.
- :defined
-
Tests are sorted in defined order.
211 212 213 |
# File 'lib/test/unit/testcase.rb', line 211 def test_order=(order) @@test_order = order end |
Instance Method Details
#==(other) ⇒ Object
It’s handy to be able to compare TestCase instances.
383 384 385 386 387 |
# File 'lib/test/unit/testcase.rb', line 383 def ==(other) return false unless(other.kind_of?(self.class)) return false unless(@method_name == other.method_name) self.class == other.class end |
#default_test ⇒ Object
363 364 365 |
# File 'lib/test/unit/testcase.rb', line 363 def default_test flunk("No tests were specified") end |
#interrupted? ⇒ Boolean
389 390 391 |
# File 'lib/test/unit/testcase.rb', line 389 def interrupted? @interrupted end |
#name ⇒ Object
Returns a human-readable name for the specific test that this instance of TestCase represents.
373 374 375 |
# File 'lib/test/unit/testcase.rb', line 373 def name "#{@method_name}(#{self.class.name})" end |
#run(result) ⇒ Object
Runs the individual test method represented by this instance of the fixture, collecting statistics, failures and errors in result.
273 274 275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 292 293 294 295 |
# File 'lib/test/unit/testcase.rb', line 273 def run(result) begin @_result = result yield(STARTED, name) begin run_setup run_test rescue Exception @interrupted = true raise unless handle_exception($!) ensure begin run_teardown rescue Exception raise unless handle_exception($!) end end result.add_run yield(FINISHED, name) ensure # @_result = nil # For test-spec's after_all :< end end |
#setup ⇒ Object
Called before every test method runs. Can be used to set up fixture information.
You can add additional setup tasks by the following code:
class TestMyClass < Test::Unit::TestCase
def setup
...
end
setup
def my_setup1
...
end
setup
def my_setup2
...
end
def test_my_class
...
end
end
Here is a call order:
-
setup
-
my_setup1
-
my_setup2
-
test_my_class
327 328 |
# File 'lib/test/unit/testcase.rb', line 327 def setup end |
#size ⇒ Object
367 368 369 |
# File 'lib/test/unit/testcase.rb', line 367 def size 1 end |
#teardown ⇒ Object
Called after every test method runs. Can be used to tear down fixture information.
You can add additional teardown tasks by the following code:
class TestMyClass < Test::Unit::TestCase
def teardown
...
end
teardown
def my_teardown1
...
end
teardown
def my_teardown2
...
end
def test_my_class
...
end
end
Here is a call order:
-
test_my_class
-
my_teardown2
-
my_teardown1
-
teardown
360 361 |
# File 'lib/test/unit/testcase.rb', line 360 def teardown end |
#to_s ⇒ Object
Overridden to return #name.
378 379 380 |
# File 'lib/test/unit/testcase.rb', line 378 def to_s name end |