Class: Test::Unit::TestCase
- Inherits:
-
Object
- Object
- Test::Unit::TestCase
- Includes:
- Assertions, Attribute, ErrorHandler, ExceptionHandler, FailureHandler, Fixture, Priority, TestCaseNotificationSupport, TestCaseOmissionSupport, TestCasePendingSupport, Util::BacktraceFilter
- Defined in:
- lib/test/unit/testcase.rb
Overview
Ties everything together. If you subclass and add your own test methods, it takes care of making them into tests and wrapping those tests into a suite. It also does the nitty-gritty of actually running an individual test and collecting its results into a Test::Unit::TestResult object.
You can run two hooks before/after a TestCase run.
Example:
class TestMyClass < Test::Unit::TestCase
class << self
def startup
...
end
def shutdown
...
end
end
def setup
...
end
def teardown
...
end
def test_my_method1
...
end
def test_my_method2
...
end
end
Here is a call order:
* startup
* setup
* test_my_method1
* teardown
* setup
* test_my_method2
* teardown
* shutdown
Constant Summary collapse
- STARTED =
name + "::STARTED"
- FINISHED =
name + "::FINISHED"
- DESCENDANTS =
[]
Constants included from Util::BacktraceFilter
Util::BacktraceFilter::TESTUNIT_FILE_SEPARATORS, Util::BacktraceFilter::TESTUNIT_PREFIX, Util::BacktraceFilter::TESTUNIT_RB_FILE
Constants included from Assertions
Constants included from ErrorHandler
ErrorHandler::PASS_THROUGH_EXCEPTIONS
Instance Attribute Summary collapse
-
#method_name ⇒ Object
readonly
Returns the value of attribute method_name.
Class Method Summary collapse
- .inherited(sub_class) ⇒ Object
-
.shutdown ⇒ Object
Called after every test case runs.
-
.startup ⇒ Object
Called before every test case runs.
-
.suite ⇒ Object
Rolls up all of the test* methods in the fixture into one suite, creating a new instance of the fixture for each method.
Instance Method Summary collapse
-
#==(other) ⇒ Object
It’s handy to be able to compare TestCase instances.
- #default_test ⇒ Object
-
#initialize(test_method_name) ⇒ TestCase
constructor
Creates a new instance of the fixture for running the test represented by test_method_name.
- #interrupted? ⇒ Boolean
-
#name ⇒ Object
Returns a human-readable name for the specific test that this instance of TestCase represents.
-
#run(result) ⇒ Object
Runs the individual test method represented by this instance of the fixture, collecting statistics, failures and errors in result.
-
#setup ⇒ Object
Called before every test method runs.
- #size ⇒ Object
-
#teardown ⇒ Object
Called after every test method runs.
-
#to_s ⇒ Object
Overridden to return #name.
Methods included from Util::BacktraceFilter
Methods included from Assertions
#assert, #assert_block, #assert_boolean, #assert_compare, #assert_const_defined, #assert_equal, #assert_fail_assertion, #assert_false, #assert_in_delta, #assert_instance_of, #assert_kind_of, #assert_match, #assert_nil, #assert_no_match, #assert_not_const_defined, #assert_not_equal, #assert_not_nil, #assert_not_same, #assert_nothing_raised, #assert_nothing_thrown, #assert_operator, #assert_raise, #assert_raise_kind_of, #assert_raise_message, #assert_raises, #assert_respond_to, #assert_same, #assert_send, #assert_throw, #assert_throws, #assert_true, #build_message, #flunk, use_pp=
Methods included from Priority
disable, enable, enabled?, included, #priority_setup, #priority_teardown
Methods included from TestCaseNotificationSupport
Methods included from TestCaseOmissionSupport
included, #omit, #omit_if, #omit_unless
Methods included from TestCasePendingSupport
Methods included from FailureHandler
Methods included from ErrorHandler
Methods included from ExceptionHandler
Methods included from Fixture
Methods included from Attribute
Constructor Details
#initialize(test_method_name) ⇒ TestCase
Creates a new instance of the fixture for running the test represented by test_method_name.
192 193 194 195 196 197 198 199 200 201 202 203 |
# File 'lib/test/unit/testcase.rb', line 192 def initialize(test_method_name) throw :invalid_test unless respond_to?(test_method_name) test_method = method(test_method_name) throw :invalid_test if test_method.arity > 0 owner = Util::MethodOwnerFinder.find(self, test_method_name) if owner.class != Module and self.class != owner throw :invalid_test end @method_name = test_method_name @test_passed = true @interrupted = false end |
Instance Attribute Details
#method_name ⇒ Object (readonly)
Returns the value of attribute method_name.
188 189 190 |
# File 'lib/test/unit/testcase.rb', line 188 def method_name @method_name end |
Class Method Details
.inherited(sub_class) ⇒ Object
90 91 92 |
# File 'lib/test/unit/testcase.rb', line 90 def inherited(sub_class) DESCENDANTS << sub_class end |
.shutdown ⇒ Object
Called after every test case runs. Can be used to tear down fixture information used in test case scope.
Here is an example test case:
class TestMyClass < Test::Unit::TestCase
class << self
def shutdown
...
end
end
def teardown
...
end
def test_my_class1
...
end
def test_my_class2
...
end
end
Here is a call order:
* test_my_class1 (or test_my_class2)
* teardown
* test_my_class2 (or test_my_class1)
* teardown
* shutdown
Note that you should not assume test order. Tests should be worked in any order.
184 185 |
# File 'lib/test/unit/testcase.rb', line 184 def shutdown end |
.startup ⇒ Object
Called before every test case runs. Can be used to set up fixture information used in test case scope.
Here is an example test case:
class TestMyClass < Test::Unit::TestCase
class << self
def startup
...
end
end
def setup
...
end
def test_my_class1
...
end
def test_my_class2
...
end
end
Here is a call order:
* startup
* setup
* test_my_class1 (or test_my_class2)
* setup
* test_my_class2 (or test_my_class1)
Note that you should not assume test order. Tests should be worked in any order.
148 149 |
# File 'lib/test/unit/testcase.rb', line 148 def startup end |
.suite ⇒ Object
Rolls up all of the test* methods in the fixture into one suite, creating a new instance of the fixture for each method.
97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 |
# File 'lib/test/unit/testcase.rb', line 97 def suite method_names = public_instance_methods(true).collect {|name| name.to_s} tests = method_names.delete_if {|method_name| method_name !~ /^test./} suite = TestSuite.new(name, self) tests.sort.each do |test| catch(:invalid_test) do suite << new(test) end end if suite.empty? catch(:invalid_test) do suite << new("default_test") end end suite end |
Instance Method Details
#==(other) ⇒ Object
It’s handy to be able to compare TestCase instances.
318 319 320 321 322 |
# File 'lib/test/unit/testcase.rb', line 318 def ==(other) return false unless(other.kind_of?(self.class)) return false unless(@method_name == other.method_name) self.class == other.class end |
#default_test ⇒ Object
298 299 300 |
# File 'lib/test/unit/testcase.rb', line 298 def default_test flunk("No tests were specified") end |
#interrupted? ⇒ Boolean
324 325 326 |
# File 'lib/test/unit/testcase.rb', line 324 def interrupted? @interrupted end |
#name ⇒ Object
Returns a human-readable name for the specific test that this instance of TestCase represents.
308 309 310 |
# File 'lib/test/unit/testcase.rb', line 308 def name "#{@method_name}(#{self.class.name})" end |
#run(result) ⇒ Object
Runs the individual test method represented by this instance of the fixture, collecting statistics, failures and errors in result.
208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 |
# File 'lib/test/unit/testcase.rb', line 208 def run(result) begin @_result = result yield(STARTED, name) begin run_setup run_test rescue Exception @interrupted = true raise unless handle_exception($!) ensure begin run_teardown rescue Exception raise unless handle_exception($!) end end result.add_run yield(FINISHED, name) ensure @_result = nil end end |
#setup ⇒ Object
Called before every test method runs. Can be used to set up fixture information.
You can add additional setup tasks by the following code:
class TestMyClass < Test::Unit::TestCase
def setup
...
end
setup
def my_setup1
...
end
setup
def my_setup2
...
end
def test_my_class
...
end
end
Here is a call order:
* setup
* my_setup1
* my_setup2
* test_my_class
262 263 |
# File 'lib/test/unit/testcase.rb', line 262 def setup end |
#size ⇒ Object
302 303 304 |
# File 'lib/test/unit/testcase.rb', line 302 def size 1 end |
#teardown ⇒ Object
Called after every test method runs. Can be used to tear down fixture information.
You can add additional teardown tasks by the following code:
class TestMyClass < Test::Unit::TestCase
def teardown
...
end
teardown
def my_teardown1
...
end
teardown
def my_teardown2
...
end
def test_my_class
...
end
end
Here is a call order:
* test_my_class
* my_teardown2
* my_teardown1
* teardown
295 296 |
# File 'lib/test/unit/testcase.rb', line 295 def teardown end |
#to_s ⇒ Object
Overridden to return #name.
313 314 315 |
# File 'lib/test/unit/testcase.rb', line 313 def to_s name end |